@@ -184,14 +184,16 @@ FALSY = ("False", "false", "No", "no", "0", False, 0, "None")
 TRUTHY = ("True", "true", "Yes", "yes", "1", True, 1)
 
 
-def _assert_col(df, col):
+def _assert_col(df, col, caller=None):
     if col not in df.columns:
+        if caller is not None:
+            exit("ph {}: Unknown column {}".format(caller, col))
         exit("Unknown column {}".format(col))
 
 
-def _assert_cols(df, cols):
+def _assert_cols(df, cols, caller=None):
     for col in cols:
-        _assert_col(df, col)
+        _assert_col(df, col, caller=caller)
 
 
 def register(fn, name=None):
@@ -272,7 +274,7 @@ def diff(*cols, periods=1, axis=0):
     if not cols:
         df = df.diff(periods=periods, axis=axis)
     else:
-        _assert_cols(df, cols)
+        _assert_cols(df, cols, "diff")
         columns = list(cols)
         df[columns] = df[columns].diff(periods=periods, axis=axis)
     pipeout(df)
@@ -409,6 +411,62 @@ def query(expr):
     pipeout(new_df)
 
 
+@register
+def grep(*expr, case=True, na=float("nan"), regex=True, column=None):
+    """Grep (with regex) for content in csv file.
+
+    Usage: cat a.csv | ph grep 0
+           cat a.csv | ph grep search_string
+           cat a.csv | ph grep "A|B"               # search hits a or b
+           cat a.csv | ph grep "a|b" --case=False  # case insensitive
+           cat a.csv | ph grep 4 --column=x
+
+    To disable regex (e.g. simple search for "." or "*" characters, use
+    --regex=False).
+
+    Search only in a specific column with --column=col.
+
+    Supports regex search queries such as "0-9A-F" and "\\d" (possibly
+    double-escaped.)
+
+    """
+    df = pipein()
+
+    if case is True or case in TRUTHY:
+        case = True
+    elif case in FALSY:
+        case = False
+    else:
+        exit("ph grep:  Unknown --case={} should be True or False".format(case))
+
+    if regex is True or regex in TRUTHY:
+        regex = True
+    elif regex in FALSY:
+        regex = False
+    else:
+        exit("ph grep:  Unknown --regex={} should be True or False".format(regex))
+
+    if column is not None:
+        _assert_col(df, column, "grep")
+
+    expr = " ".join(str(e) for e in expr)  # force string input
+
+    try:
+        import numpy
+    except ImportError:
+        exit("numpy needed for grep.  pip install numpy")
+
+    retval = df[
+        numpy.logical_or.reduce(
+            [
+                df[col].astype(str).str.contains(expr, case=case, na=na, regex=regex)
+                for col in (df.columns if column is None else [column])
+            ]
+        )
+    ]
+    pipeout(retval)
+
+
 @register
 def appendstr(col, s, newcol=None):
     """Special method to append a string to the end of a column.
@@ -425,7 +483,7 @@ def appendstr(col, s, newcol=None):
 @register
 def split(col, pat=" "):
     df = pipein()
-    _assert_col(df, col)
+    _assert_col(df, col, "split")
     df[[col, col + "_rhs"]] = df[col].str.split(pat=pat, n=1, expand=True)
     pipeout(df)
 
@@ -444,7 +502,7 @@ def strip(*cols, lstrip=False, rstrip=False):
         cols = list(df.columns)
     else:
         cols = list(cols)
-    _assert_cols(df, cols)
+    _assert_cols(df, cols, "strip")
     for c in cols:
         if lstrip in TRUTHY:
             df[c] = df[c].str.lstrip()
@@ -455,6 +513,44 @@ def strip(*cols, lstrip=False, rstrip=False):
     pipeout(df)
 
 
+@register
+def removeprefix(col, prefix=" "):
+    """Remove prefix of contents of a column.
+
+    Usage: cat a.csv | ph removeprefix col1 ..
+
+    See also @removesuffix @strip
+
+    """
+    prefix = str(prefix)
+    plen = len(prefix)
+    df = pipein()
+    _assert_col(df, col, "removeprefix")
+    df[col] = df[col].apply(
+        lambda s: str(s)[plen:] if str(s).startswith(prefix) else str(s)
+    )
+    pipeout(df)
+
+
+@register
+def removesuffix(col, suffix=" "):
+    """Remove suffix of contents of a column.
+
+    Usage: cat a.csv | ph removesuffix col1 ..
+
+    See also @removeprefix @strip
+
+    """
+    suffix = str(suffix)
+    plen = len(suffix)
+    df = pipein()
+    _assert_col(df, col, "removesuffix")
+    df[col] = df[col].apply(
+        lambda s: str(s)[:-plen] if str(s).endswith(suffix) else str(s)
+    )
+    pipeout(df)
+
+
 @register
 def astype(type, column=None, newcolumn=None):
     """Cast a column to a different type.
@@ -548,7 +644,7 @@ def groupby(*columns, how="sum", as_index=False):
     if not columns:
         exit("Needs at least one column to group by")
     df = pipein()
-    _assert_cols(df, columns)
+    _assert_cols(df, columns, "groupby")
     if as_index in TRUTHY:
         as_index = True
     elif as_index in FALSY:
@@ -578,7 +674,7 @@ def rolling(window, *columns, how="sum", win_type=None, std=None, beta=None, tau
     df = pipein()
     orig_columns = list(df.columns)
     columns = list(columns)
-    _assert_cols(df, columns)
+    _assert_cols(df, columns, "rolling")
 
     if not columns:
         columns = list(df.columns)
@@ -598,6 +694,10 @@ def rolling(window, *columns, how="sum", win_type=None, std=None, beta=None, tau
         retval = fn()
 
     df = pd.concat([retval, nonrollin], axis=1)
+    for col in orig_columns:
+        if col not in df.columns:
+            op = "ph rolling"
+            exit('{}: Could not perform rolling window on column "{}"'.format(op, col))
     df = df[orig_columns]
     pipeout(df)
 
@@ -768,7 +868,9 @@ def plot(*args, **kwargs):
             ph plot --logx=True
             ph plot --logy=True
             ph plot --loglog=True
-
+            ph plot --savefig=fname.png
+            ph plot --savefig=fname.svg
+            ph plot --savefig=fname.svg --savefig-dpi=300
     """
     try:
         import matplotlib.pyplot as plt
@@ -778,14 +880,24 @@ def plot(*args, **kwargs):
     df = pipein()
     index = kwargs.get("index")
     if index is not None:
+        _assert_col(df, index, caller="plot")
         df = df.set_index(index)
         del kwargs["index"]
     for log_ in ("logx", "logy", "loglog"):
         if kwargs.get(log_) in TRUTHY:
             kwargs[log_] = True
-
+    fname = kwargs.get("savefig")
+    dpi = kwargs.get("savefig-dpi")
+    if fname:
+        del kwargs["savefig"]
+    if dpi:
+        del kwargs["savefig-dpi"]
     df.plot(**kwargs)
-    plt.show()
+    if fname:
+        plt.tight_layout()
+        plt.savefig(fname, dpi=dpi)
+    else:
+        plt.show()
     pipeout(df)
 
 
@@ -859,7 +971,7 @@ def date(col=None, unit=None, origin="unix", errors="raise", dayfirst=False, **k
         if col is None:
             df = pd.to_datetime(df, unit=unit, origin=origin, errors=errors)
         else:
-            _assert_col(df, col)
+            _assert_col(df, col, "date")
             if date_parser is None:
                 df[col] = pd.to_datetime(
                     df[col], unit=unit, origin=origin, errors=errors, dayfirst=dayfirst
@@ -1422,7 +1534,7 @@ def columns(*cols, **kwargs):
             if col.endswith(q) and col not in cols:
                 cols.append(col)
 
-    _assert_cols(df, cols)
+    _assert_cols(df, cols, "columns")
 
     if not cols and not kwargs:
         print("columns")
@@ -1451,7 +1563,7 @@ def spencer(*cols):
             yield sum(seq[i] * _SPENCER[i] / _SPENCER_SUM for i in range(15))
 
     df = pipein()
-    _assert_cols(df, cols)
+    _assert_cols(df, cols, "spencer")
     prefix = [float("nan")] * 7
     suffix = [float("nan")] * 8
     if not cols:
@@ -1531,7 +1643,7 @@ def drop(*columns, **kwargs):
     if kwargs.get("axis") in (None, 0, "index"):
         columns = [__tryparse(col) for col in columns]
     elif kwargs.get("axis") in (1, "columns"):
-        _assert_cols(df, columns)
+        _assert_cols(df, columns, "drop")
     else:
         exit(
             "--axis=index (or 0) or --axis=columns (or 1), not {}".format(
@@ -1582,7 +1694,7 @@ def sort(col):
 
     """
     df = pipein()
-    _assert_col(df, col)
+    _assert_col(df, col, "sort")
     pipeout(df.sort_values(col))
 
 
@@ -1598,7 +1710,7 @@ def polyfit(x, y, deg=1):
 
     """
     df = pipein()
-    _assert_cols(df, (x, y))
+    _assert_cols(df, (x, y), "polyfit")
     deg = __tryparse(deg)
     if not isinstance(deg, int) or deg <= 0:
         exit("deg={} should be a positive int".format(deg))
@@ -1637,7 +1749,7 @@ def _main(argv):
     if cmd in ("-v", "--version"):
         print_version()
         exit()
-    if cmd in ("--h", "--help"):
+    if cmd in ("-h", "--h", "--help"):
         cmd = "help"
     if cmd not in COMMANDS:
         exit("Unknown command {}.".format(cmd))
